########################################################
# This code produces CWM trait-environment models at the local (LOC) scale.
# Code produces response curves, calculates model fit and relative importance of the predictors,
# as well as produces spatial predictions and model evaluations.
#
# Analyses used in paper: Plant trait-environment relationships in tundra are consistent across spatial scales
# By: Tuuli Rissanen, Pekka Niittynen, Janne Soininen, Anna-Maria Virkkala and Miska Luoto
## email: tuuli.rissanen@helsinki.fi
# Modified and deposited: 2023-03-27
#
##############################################################################################################
library(data.table); library(tidyverse)
library(viridis); library(egg)
library(scales); library(robustbase)
library(raster); library(rgdal)
library(modEvA); library(matrixStats) 
library(corrplot); library(MASS)

# Set working directory
setwd("./Data")

# Read in local modelling data
d_loc <- read.csv("Modelling_data_LOC.csv")
str(d_loc)
head(d_loc)

# Read in environmental variables covering all three scales. From these data we extract 
# the range for the response curves.
env_all <- read.csv("Envvars_all_scales.csv")
head(env_all)

# Read in LOC raster stack for spatial predictions
rstack <- stack("EnvarStack_LOC.tif")
names(rstack) <- c("Temp","Snow","TPI","pH")
plot(rstack)

# Correlation matrix of the environmental variables
corrplot(cor(d_loc[,7:10], method="spearman"), method="number") 

# Name trait and environmental variables
traits <- c("Height", "Seed_mass", "SLA")
predictors <- c("Temp","Snow","TPI","pH")

# Set environmental variables at the LOC scale
envvars <- d_loc[,7:10] 
head(envvars)

##############################################################
# MODELLING
mods <- list() # store AIC based model for each trait
aics <- as.data.frame(matrix(ncol=3, nrow=3)) # df for AIC values
colnames(aics) <- c("Trait", "aic_full","aic_final")
aics$Trait <- traits

SP <- list() # list to store spatial predictions
RC <- list() # list to store response curves

# Choose model based on AIC and calculate response curves and make spatial predictions
for (i in traits){
  # Full model
  m_full <- glm(d_loc[[i]] ~ Temp + I(Temp^2) + Snow + I(Snow^2) + TPI + I(TPI^2) + pH + I(pH^2),
                data=d_loc , family="Gamma"(link="log"))
  aics[aics$Trait==i,"aic_full"] <- m_full$aic
  
  # Model selection based on AIC value
  m_aic <- stepAIC(m_full, trace=1)
  aics[aics$Trait==i,"aic_final"] <- m_aic$aic
  mods[[i]] <- m_aic
  
  # Spatial predictions
  sp <- predict(object=rstack,model=m_aic, fun=predict, type="response")
  SP[[i]] <- sp
  
  # Response curves
  rc <- as.data.frame(matrix(ncol=1, nrow=100))
  colnames(rc) <- "Trait" 
  rc$Trait <- i
  
  rc_temp <- as.data.frame(matrix(ncol=4, nrow=100))
  colnames(rc_temp) <- predictors
  
  preds_minmax <- as.data.frame(matrix(ncol=4, nrow=100)) 
  colnames(preds_minmax) <- predictors
  
  for (p in predictors) {
    print(p)
    predsMed <- data.frame(matrix(as.numeric(colMedians(as.matrix(envvars[,1:4]))),  
                                  nrow = 100, ncol=4, byrow = TRUE, dimnames = list(1:100, names(envvars[,1:4]))))
    # Range of TPI is set based on LOC scale
    if (p == "TPI"){
      predsMed[,p] <- seq(from=min(envvars[,p]), to=max(envvars[,p]), length.out = 100)
      rc_temp[,p] <- predict.glm(m_aic, newdata=predsMed, type="response")
      preds_minmax[,p] <- predsMed[,p]
    # Range of other predictors is set based on min and max values over all three scales
    } else {
      predsMed[,p] <- seq(from=min(env_all[,p]), to=max(env_all[,p]), length.out = 100)
      rc_temp[,p] <- predict.glm(m_aic, newdata=predsMed, type="response")
      preds_minmax[,p] <- predsMed[,p]
    }
  }
  colnames(preds_minmax) <- paste0(predictors,"_obs")
  rc <- bind_cols(rc,rc_temp)
  rc <- bind_cols(rc,preds_minmax)
  RC[[i]] <- rc
}

# Save models, response curves and spatial predictions
saveRDS(mods,"./mods_LOC.rds")
saveRDS(RC, "./RC_LOC.rds")
saveRDS(SP,"./SP_LOC.rds")

# Calculate model fit (explained deviance) and variable importance using 10 time data sampling
reps=10
dev_expl <- as.data.frame(matrix(nrow=reps, ncol=3)) # df for D2
colnames(dev_expl) <- traits

varimp <- as.data.frame(matrix(nrow=0, ncol=length(predictors)+1)) # df for varimp
colnames(varimp) <- c("Trait",predictors)

# Models with 10 reps using sampled data
for (i in traits){
# Create df for varimp
  vi <- as.data.frame(matrix(ncol=5,nrow=reps))
  colnames(vi) <- c("Trait", predictors)
  vi$Trait <- i
    for(r in 1:reps){
      # Sample data
      bs <- d_loc[(sample(nrow(d_loc),replace=TRUE)),]
      preds <- bs[,7:10]
      # Set trait-specific models
        if (i == "Height"){
          m <- glm(bs[[i]] ~ Temp + I(Temp^2) + pH, # Height
             data=bs , family="Gamma"(link="log"))
        } else if ( i == "Seed_mass"){
          m <- glm(bs[[i]] ~ Temp + I(Temp^2) + Snow, # Seed mass
           data=bs , family="Gamma"(link="log"))
        } else {
          m <- glm(bs[[i]] ~ I(Snow^2),
             data=bs , family="Gamma"(link="log")) # SLA
  }

  # Extract explained deviance
  dev_expl[r,i] <- Dsquared(m)*100
  
  # Calculate variable importance based on change in R2 
  rel.inf_temp <- c()
  
  for (p in predictors){
    preds_shuf <- preds
    R2_shuf <- c()
    for (t in 1:10) {
      preds_shuf[,p] <- sample(preds[,p])
      
      if (i == "Height") {
        m_suf <- glm(bs[[i]] ~ Temp + I(Temp^2) + pH,
                   data=data.frame(bs[[i]],preds_shuf) , family="Gamma"(link="log")) # Height
      } else if (i == "Seed_mass") {
        m_suf <- glm(bs[[i]] ~ Temp + I(Temp^2) + Snow,
                    data=data.frame(bs[[i]],preds_shuf) , family="Gamma"(link="log")) # Seed mass
      } else {
        m_suf <- glm(bs[[i]] ~ I(Snow^2),
                     data=data.frame(bs[[i]],preds_shuf) , family="Gamma"(link="log")) # SLA
      }
      
      R2_shuf[t] <- Dsquared(m_suf)*100
      print(R2_shuf[t])
    } 
    rel.inf_temp[p] <- mean(dev_expl[r,i]-R2_shuf)
  }
  rel.inf_temp[rel.inf_temp<0] <- 0
  vi[r,2:5] <- (rel.inf_temp/sum(rel.inf_temp))*100
  
}
varimp <- rbind(varimp, vi)
}

# Save D2 and varimp
saveRDS(dev_expl,"./dev_expl_LOC.rds")
saveRDS(varimp, "./varimp_LOC.rds")

##############################################################################################
# Model evaluation using leave-one-out cross-validation
obs <- nrow(d_loc)
res1 <- NULL
res2 <- NULL
res3 <- NULL

# Data frame to store correlations between observed and predicted values 
cors <- as.data.frame(matrix(nrow=0, ncol=3))
colnames(cors) <- c("cor_height", "cor_sm", "cor_sla")

# Data frame to store observed and predicted trait values
# RMSE can be calculated based on this data frame
obs_pred <- as.data.frame(matrix(nrow=obs, ncol=6))

for (i in 1:obs){
  print(i)
  d_eval <- d_loc[i,]
  d_cal <- d_loc[-i,]
  
  m_h <- glm(Height ~ Temp + I(Temp^2) + pH, # Height
             data=d_cal , family="Gamma"(link="log"))
  m_sm <- glm(Seed_mass ~ Temp + I(Temp^2) + Snow, # Seed mass
              data=d_cal , family="Gamma"(link="log"))
  m_sla <- glm(SLA ~ I(Snow^2),
               data=d_cal , family="Gamma"(link="log")) # SLA
  
  h_pred <- predict.glm(m_h, newdata = d_eval, type="response")
  sm_pred <- predict.glm(m_sm, newdata = d_eval, type="response")
  sla_pred <- predict.glm(m_sla, newdata = d_eval, type="response")
  
  res1[i] <- as.numeric(h_pred)
  res2[i] <- as.numeric(sm_pred)
  res3[i] <- as.numeric(sla_pred)
}

cors[1,"cor_height"] <- cor(res1, d_loc$Height)
cors[1,"cor_sm"] <- cor(res2, d_loc$Seed_mass)
cors[1,"cor_sla"] <- cor(res3, d_loc$SLA)

obs_pred$V1 <- d_loc$Height
obs_pred$V2 <- d_loc$Seed_mass
obs_pred$V3 <- d_loc$SLA
obs_pred$V4 <- res1
obs_pred$V5 <- res2
obs_pred$V6 <- res3
names(obs_pred) <- c("H_obs","SM_obs","SLA_obs","H_pred","SM_pred","SLA_pred")

write.table(cors,"./cors_LOC.csv", sep=",", dec=".", row.names = FALSE)
write.table(obs_pred,"./obs_pred_traits_LOC.csv", sep=",", dec=".", row.names = FALSE)


