########################################################
# This code produces CWM trait-environment models at the subcontinental (SUB) scale.
# Code produces response curves, calculates model fit and relative importance of the predictors,
# as well as produces spatial predictions and model evaluations.
#
# Analyses used in paper: Plant trait-environment relationships in tundra are consistent across spatial scales
# By: Tuuli Rissanen, Pekka Niittynen, Janne Soininen, Anna-Maria Virkkala and Miska Luoto
## email: tuuli.rissanen@helsinki.fi
# Modified and deposited: 2023-03-27
#
################################################################################################################
library(data.table); library(tidyverse)
library(viridis); library(egg)
library(scales); library(robustbase)
library(raster); library(rgdal)
library(modEvA); library(matrixStats) 
library(corrplot); library(MASS)

# Set working directory
setwd("./Data")

# Read in subcontinental modelling data
d_sub <- read.csv("Modelling_data_SUB.csv")
str(d_sub)
head(d_sub)
summary(d_sub)

# Read in environmental variables covering all three scales. From these data we extract 
# the range for the response curves.
env_all <- read.csv("Envvars_all_scales.csv")
head(env_all)

# Read in SUB raster stack for spatial predictions
rstack <- stack("EnvarStack_SUB.tif")
names(rstack) <- c("Temp","Snow","TPI","pH")
plot(rstack)

# Correlation matrix of the environmental variables
corrplot(cor(d_sub[,7:10], method="spearman"), method = "number") 

# Name trait and environmental variables
traits <- c("Height", "Seed_mass", "SLA")
predictors <- c("tavg_july","snow_doy","TPI","pH")

# Set environmental variables
envvars <- d_sub[,c(7:10)] 
head(envvars)

##############################################################
# MODELLING
mods <- list() # store AIC based model for each trait
aics <- as.data.frame(matrix(ncol=3, nrow=3)) # df for AIC values
colnames(aics) <- c("Trait", "aic_full","aic_final")
aics$Trait <- traits

SP <- list() # list to store spatial predictions
RC <- list() # list to store response curves

# Choose model based on AIC and calculate response curves and make spatial predictions
# Run separately for each trait as for Seed mass stepAIC does not work if all terms are kept 
for (i in traits){
  # Full model
  m_full <- glm(d_sub[[i]] ~ tavg_july + I(tavg_july^2) + snow_doy + I(snow_doy^2) + TPI + I(TPI^2) + pH + I(pH^2),
                data=d_sub , family="Gamma"(link="log"))
  aics[aics$Trait==i,"aic_full"] <- m_full$aic
  
  # Model selection based on AIC value
  m_aic <- stepAIC(m_full, trace=1)
  #m_aic <- m_full # needed for Seed mass, were all terms kept in the model
  aics[aics$Trait==i,"aic_final"] <- m_aic$aic
  mods[[i]] <- m_aic
  
  # Spatial predictions
  sp <- predict(object=rstack,model=m_aic, fun=predict, type="response")
  SP[[i]] <- sp
  
  # Response curves
  rc <- as.data.frame(matrix(ncol=1, nrow=100))
  colnames(rc) <- "Trait" 
  rc$Trait <- i
  
  rc_temp <- as.data.frame(matrix(ncol=4, nrow=100))
  colnames(rc_temp) <- predictors
  
  preds_minmax <- as.data.frame(matrix(ncol=4, nrow=100)) 
  colnames(preds_minmax) <- predictors
  
  for (p in predictors) {
    predsMed <- data.frame(matrix(as.numeric(colMedians(as.matrix(envvars[,1:4]))),  
                                  nrow = 100, ncol=4, byrow = TRUE, dimnames = list(1:100, names(envvars[,1:4]))))
    predsMed[,p] <- seq(from=min(env_all[,p]), to=max(env_all[,p]), length.out = 100)
    rc_temp[,p] <- predict.glm(m_aic, newdata=predsMed, type="response")
    preds_minmax[,p] <- predsMed[,p]
  }
  colnames(preds_minmax) <- paste0(predictors,"_obs")
  rc <- bind_cols(rc,rc_temp)
  rc <- bind_cols(rc,preds_minmax)
  RC[[i]] <- rc
}

# Save models, response curves and spatial predictions
saveRDS(aics, "./aics_sub.rds")
saveRDS(mods,"./mods_sub.rds")
saveRDS(RC, "./RC_sub.rds")
saveRDS(SP,"./SP_sub.rds")

# Calculate explained deviance and variable importance using 10 time data sampling
reps=10
dev_expl <- as.data.frame(matrix(nrow=reps, ncol=3)) # df for D2
colnames(dev_expl) <- traits

varimp <- as.data.frame(matrix(nrow=0, ncol=length(predictors)+1)) # df for varimp
colnames(varimp) <- c("Trait",predictors)

# Models with 10 reps using sampled data
# Needs to be run separately for each trait due to specific models
i <- "Height"
#i <- "Seed_mass"
#i <- "SLA"

for (i in traits){
  # Create df for varimp
  vi <- as.data.frame(matrix(ncol=5,nrow=reps))
  colnames(vi) <- c("Trait", predictors)
  vi$Trait <- i
  for(r in 1:reps){
    # Sample data
    bs <- d_sub[(sample(nrow(d_sub),replace=TRUE)),]
    preds <- bs[,7:10]
    # Take trait specific model 
    if (i == "Height"){
      m <- glm(bs[[i]] ~ I(tavg_july^2) + snow_doy + I(snow_doy^2) + TPI + pH + I(pH^2), # Height
             data=bs , family="Gamma"(link="log"))
      } else if (i == "Seed_mass"){
        m <- glm(bs[[i]] ~ I(tavg_july^2) + snow_doy + I(snow_doy^2) + pH + I(pH^2), # Seed mass
         data=bs , family="Gamma"(link="log"))
      } else {
        m <- glm(bs[[i]] ~ I(tavg_july^2) + snow_doy + I(snow_doy^2) + TPI + pH + I(pH^2),
          data=bs , family="Gamma"(link="log")) # SLA
    }
  print(m$formula)
    # Extract explained deviance
    dev_expl[r,i] <- Dsquared(m)*100
  
  # Calculate variable importance based on change in R2 
  rel.inf_temp <- c()
  
  for (p in predictors){
    preds_shuf <- preds
    R2_shuf <- c()
    for (t in 1:10) {
      preds_shuf[,p] <- sample(preds[,p])
      if (i == "Height"){
        m_suf <- glm(bs[[i]] ~ I(tavg_july^2) + snow_doy + I(snow_doy^2) + TPI + pH + I(pH^2),
                     data=data.frame(bs[[i]],preds_shuf) , family="Gamma"(link="log")) # Height
      } else if (i == "Seed_mass"){
        m_suf <- glm(bs[[i]] ~ I(tavg_july^2) + snow_doy + I(snow_doy^2) + pH + I(pH^2),
                 data=data.frame(bs[[i]],preds_shuf) , family="Gamma"(link="log")) # Seed mass
      } else {
        m_suf <- glm(bs[[i]] ~ I(tavg_july^2) + snow_doy + I(snow_doy^2) + TPI + pH + I(pH^2),
             data=data.frame(bs[[i]],preds_shuf) , family="Gamma"(link="log")) # SLA
      }
      
      R2_shuf[t] <- Dsquared(m_suf)*100
      print(R2_shuf[t])
    } 
    rel.inf_temp[p] <- mean(dev_expl[r,i]-R2_shuf)
  }
  rel.inf_temp[rel.inf_temp<0] <- 0
  vi[r,2:5] <- (rel.inf_temp/sum(rel.inf_temp))*100
  
}
varimp <- rbind(varimp, vi)
}

summary(dev_expl)
summary(varimp)

# Save D2 and varimp
saveRDS(dev_expl,"./dev_expl_sub.rds")
saveRDS(varimp, "./varimp_sub.rds")

##################################################################
# MODEL EVALUATION

# Cross-validation (70 / 30)
obs <- nrow(d_sub)
ncal <- round((0.7*obs),0)
cvs <- as.data.frame(matrix(nrow=10, ncol=3))
colnames(cvs) <- c("cor_height", "cor_sm", "cor_sla")

h_eval <- as.data.frame(matrix(nrow = 0, ncol=3))
names(h_eval) <- c("h_obs","h_pred","i")
sm_eval <- as.data.frame(matrix(nrow = 0, ncol=3))
names(sm_eval) <- c("sm_obs","sm_pred","i")
sla_eval <- as.data.frame(matrix(nrow = 0, ncol=3))
names(sla_eval) <- c("sm_obs","sla_pred","i")

for (i in 1:10){
  ds <- sample(obs, ncal, replace=FALSE)
  d_cal <- d_sub[ds,]
  d_eval <- d_sub[-ds,]
  m_h <- glm(Height ~ I(tavg_july^2) + snow_doy + I(snow_doy^2) + TPI + pH + I(pH^2), # Height
             data=d_cal , family="Gamma"(link="log"))
  m_sm <- glm(Seed_mass ~ I(tavg_july^2) + snow_doy + I(snow_doy^2) + pH + I(pH^2), # Seed mass
              data=d_cal , family="Gamma"(link="log"))
  m_sla <- glm(SLA ~ I(tavg_july^2) + snow_doy + I(snow_doy^2) + TPI + pH + I(pH^2),
               data=d_cal , family="Gamma"(link="log")) # SLA
  
  h_pred <- predict.glm(m_h, newdata=d_eval, type="response")
  sm_pred <- predict.glm(m_sm, newdata=d_eval, type="response")
  sla_pred <- predict.glm(m_sla, newdata=d_eval, type="response")
  
  h_e <- cbind.data.frame(d_eval$Height, h_pred)
  h_e$i <- i
  names(h_e) <- c("h_obs","h_pred","i")
  sm_e <- cbind.data.frame(d_eval$Seed_mass, sm_pred)
  sm_e$i <- i
  names(sm_e) <- c("sm_obs","sm_pred","i")
  sla_e <- cbind.data.frame(d_eval$SLA, sla_pred)
  sla_e$i <- i
  names(sla_e) <- c("sla_obs","sla_pred","i")
  
  cvs[i,"cor_height"] <- cor(h_pred, d_eval$Height)
  cvs[i,"cor_sm"] <- cor(sm_pred, d_eval$Seed_mass)
  cvs[i,"cor_sla"] <- cor(sla_pred, d_eval$SLA)
  
  h_eval <- rbind.data.frame(h_eval,h_e)
  sm_eval <- rbind.data.frame(sm_eval,sm_e)
  sla_eval <- rbind.data.frame(sla_eval,sla_e)
}

summary(cvs)
boxplot(cvs)
cvs$scl <- "sub"

plot(h_eval$h_obs,h_eval$h_pred)
cor(h_eval$h_obs, h_eval$h_pred)

plot(sm_eval$sm_obs,sm_eval$sm_pred)
cor(sm_eval$sm_obs, sm_eval$sm_pred)

plot(sla_eval$sla_obs,sla_eval$sla_pred)
cor(sla_eval$sla_obs,sla_eval$sla_pred)

write.table(cvs, "./CVs.csv", row.names = FALSE, sep=",", dec=".")
write.table(h_eval, "./H_eval.csv", row.names = FALSE, sep=",", dec=".")
write.table(sm_eval, "./SM_eval.csv", row.names = FALSE, sep=",", dec=".")
write.table(sla_eval, "./SLA_eval.csv", row.names = FALSE, sep=",", dec=".")

